/* {=================================================================
 *
 * utils.c
 * A few utility functions for PostBio
 * Author: Luis Carvalho <lexcarvalho at gmail dot com>
 * Please check copyright notice at the bottom of postbio.source
 * $Id: postbio.c,v 1.1 2008-01-04 15:32:51 carvalho Exp $
 *
 * ==================================================================} */

#include <postgres.h>
#include <fmgr.h>

#ifdef PG_MODULE_MAGIC
PG_MODULE_MAGIC;
#endif

/* {=====================================================================
 *    Prototypes
 * ======================================================================} */

/* int_interval */
Datum int_interval_in (PG_FUNCTION_ARGS);
Datum int_interval_out (PG_FUNCTION_ARGS);
Datum int_text (PG_FUNCTION_ARGS);
Datum int_low (PG_FUNCTION_ARGS);
Datum int_high (PG_FUNCTION_ARGS);
Datum integer_to_int (PG_FUNCTION_ARGS);
Datum intpair_to_int (PG_FUNCTION_ARGS);
Datum int_interval_compare (PG_FUNCTION_ARGS);
Datum int_interval_lt (PG_FUNCTION_ARGS);
Datum int_interval_le (PG_FUNCTION_ARGS);
Datum int_interval_eq (PG_FUNCTION_ARGS);
Datum int_interval_ge (PG_FUNCTION_ARGS);
Datum int_interval_gt (PG_FUNCTION_ARGS);
Datum int_interval_left (PG_FUNCTION_ARGS);
Datum int_interval_overleft (PG_FUNCTION_ARGS);
Datum int_interval_overlap (PG_FUNCTION_ARGS);
Datum int_interval_overright (PG_FUNCTION_ARGS);
Datum int_interval_right (PG_FUNCTION_ARGS);
Datum int_interval_same (PG_FUNCTION_ARGS);
Datum int_interval_contain (PG_FUNCTION_ARGS);
Datum int_interval_contained (PG_FUNCTION_ARGS);
Datum g_int_interval_consistent (PG_FUNCTION_ARGS);
Datum g_int_interval_union (PG_FUNCTION_ARGS);
Datum g_int_interval_compress (PG_FUNCTION_ARGS);
Datum g_int_interval_decompress (PG_FUNCTION_ARGS);
Datum g_int_interval_penalty (PG_FUNCTION_ARGS);
Datum g_int_interval_picksplit (PG_FUNCTION_ARGS);
Datum g_int_interval_same (PG_FUNCTION_ARGS);
Datum int_distance_low (PG_FUNCTION_ARGS);
Datum int_distance_high (PG_FUNCTION_ARGS);
Datum int_distance_center (PG_FUNCTION_ARGS);
Datum int_distance (PG_FUNCTION_ARGS);
Datum int_join (PG_FUNCTION_ARGS);
Datum int_overlap (PG_FUNCTION_ARGS);
Datum int_size (PG_FUNCTION_ARGS);
Datum int_buffer1 (PG_FUNCTION_ARGS);
Datum int_buffer2 (PG_FUNCTION_ARGS);
Datum int_span (PG_FUNCTION_ARGS);
/* mum */
Datum mum (PG_FUNCTION_ARGS);
/* utils */
Datum revcomp (PG_FUNCTION_ARGS);

/* {=====================================================================
 *    Functions
 * ======================================================================} */

/* Watson-Crick complement */
static char wccomp (char c) {
  switch (c) {
    case 'a': return 't';
    case 'c': return 'g';
    case 'g': return 'c';
    case 't': return 'a';
    case 'r': return 'y'; /* purine: a or g */
    case 'y': return 'r'; /* pyrimidine: c or t */
    case 's': case 'w': return c; /* strong (c or g) or weak (a or t) */
    case 'm': return 'k'; /* a or c */
    case 'k': return 'm'; /* g or t */
    case 'b': return 'v'; /* c, g or t */
    case 'd': return 'h'; /* a, g or t */
    case 'h': return 'd'; /* a, c or t */
    case 'v': return 'b'; /* a, c or g */
    default: return 'n';
  }
}

PG_FUNCTION_INFO_V1(revcomp);
Datum revcomp (PG_FUNCTION_ARGS) {
  text *str = PG_GETARG_TEXT_P(0);
  bool comp;
  text *rstr;
  char *ps, *pr;
  size_t l = VARSIZE(str) - VARHDRSZ;
  rstr = (text *) palloc(VARSIZE(str));
  VARATT_SIZEP(rstr) = VARSIZE(str);
  ps = VARDATA(str);
  pr = VARDATA(rstr);
  comp = PG_ARGISNULL(1);
  if (!comp) comp = PG_GETARG_BOOL(1);
  if (comp)
    while (l--) *pr++ = wccomp(ps[l]);
  else
    while (l--) *pr++ = ps[l];
  PG_RETURN_TEXT_P(rstr);
}

